from pydantic import BaseModel

from cmk_dev_site.saas.constants import TENANT_ID, URL


class OIDCConfig(BaseModel):
    base_url: str = "http://localhost"
    client_id: str = "notused"
    tenant_id: str = TENANT_ID
    well_known: str = f"{URL}/.well-known/openid-configuration"
    saas_api_url: str = URL
    logout_url: str = f"{URL}/logout"


class AdminPanelUrlConfig(BaseModel):
    uap_url: str = "https://admin-panel.saas-prod.cloudsandbox.checkmk.cloud/"
    bug_tracker_url: str = "https://admin-panel.saas-prod.cloudsandbox.checkmk.cloud/bug-report"
    download_agent_user: str = "automation"
    tenant_id: str = TENANT_ID
